#!/usr/bin/env python

import os
import sys

import django

from django.conf import settings
from django.test.runner import DiscoverRunner


DEFAULT_SETTINGS = dict(
    INSTALLED_APPS=[
        'django.contrib.auth',
        'django.contrib.contenttypes'
    ],
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'test'
        }
    },
    GRAPHENE={
        'SCHEMA': 'tests.schema.schema',
    },
    AUTHENTICATION_BACKENDS=[
        'graphql_jwt.backends.JWTBackend'
    ],
    ROOT_URLCONF='tests.urls'
)


def runtests():
    if not settings.configured:
        settings.configure(**DEFAULT_SETTINGS)

    django.setup()

    parent = os.path.dirname(os.path.abspath(__file__))
    sys.path.insert(0, parent)

    failures = DiscoverRunner(
        verbosity=1,
        interactive=True,
        failfast=False)\
        .run_tests(['tests'])

    sys.exit(failures)


if __name__ == '__main__':
    runtests()



import graphene

from calendar import timegm
from datetime import datetime

from graphene.types.generic import GenericScalar

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from . import settings

from .utils import (
    jwt_encode, jwt_payload,
    get_payload, get_user_by_payload
)


class JWTMutationMixin(object):

    class Arguments:
        token = graphene.String()


class Verify(JWTMutationMixin, graphene.Mutation):
    payload = GenericScalar()

    @classmethod
    def mutate(cls, root, info, token, **args):
        payload = get_payload(token)
        return cls(payload=payload)


class Refresh(JWTMutationMixin, graphene.Mutation):
    data = GenericScalar()

    @classmethod
    def mutate(cls, root, info, token, **args):
        payload = get_payload(token)
        user = get_user_by_payload(payload)
        orig_iat = payload.get('orig_iat')

        if settings.JWT_VERIFY_REFRESH_EXPIRATION:
            if orig_iat:
                utcnow = timegm(datetime.utcnow().utctimetuple())
                expiration = orig_iat +\
                    settings.JWT_REFRESH_EXPIRATION_DELTA.total_seconds()

                if utcnow > expiration:
                    raise ValidationError(_('Refresh has expired'))
            else:
                raise ValidationError(_('orig_iat field is required'))

        refresh_payload = jwt_payload(user)
        refresh_payload['orig_iat'] = orig_iat

        return cls(data={
            'token': jwt_encode(refresh_payload),
            'payload': payload
        })



import json

from unittest.mock import patch, MagicMock

from django.http import JsonResponse
from graphql_jwt.middleware import JWTMiddleware

from .base import GraphQLJWTTestCase


class MiddlewareTests(GraphQLJWTTestCase):

    def setUp(self):
        super().setUp()

        self.get_response_mock = MagicMock(return_value=JsonResponse({}))
        self.middleware = JWTMiddleware(self.get_response_mock)

    def test_authenticate(self):
        headers = {
            'HTTP_AUTHORIZATION': 'JWT ' + self.token,
        }

        request = self.factory.get('/', **headers)
        self.middleware(request)

        self.get_response_mock.assert_called_once_with(request)

    @patch('graphql_jwt.middleware.authenticate', return_value=None)
    def test_user_not_authenticate(self, *args):
        headers = {
            'HTTP_AUTHORIZATION': 'JWT ' + self.token,
        }

        request = self.factory.get('/', **headers)
        self.middleware(request)

        self.get_response_mock.assert_called_once_with(request)

    def test_graphql_error(self):
        headers = {
            'HTTP_AUTHORIZATION': 'JWT invalid',
        }

        request = self.factory.get('/', **headers)
        response = self.middleware(request)
        content = json.loads(response.content.decode('utf-8'))

        self.assertTrue(content['errors'])
        self.get_response_mock.assert_not_called()

    def test_header_not_found(self):
        request = self.factory.get('/')
        self.middleware(request)

        self.get_response_mock.assert_called_once_with(request)

    def test_user_is_authenticated(self):
        headers = {
            'HTTP_AUTHORIZATION': 'JWT ' + self.token,
        }

        request = self.factory.get('/', **headers)
        request.user = self.user
        self.middleware(request)

        self.get_response_mock.assert_called_once_with(request)



import graphene
import graphql_jwt


from django.contrib.auth import get_user_model
from graphene_django import DjangoObjectType


class UserNode(DjangoObjectType):

    class Meta:
        model = get_user_model()


class Query(graphene.ObjectType):
    user = graphene.Field(UserNode)


class Mutations(graphene.ObjectType):
    verify_token = graphql_jwt.Verify.Field()
    refresh_token = graphql_jwt.Refresh.Field()


schema = graphene.Schema(query=Query, mutation=Mutations)



from .utils import (
    get_authorization_header,
    get_user_by_token,
    get_user_by_natural_key
)


class JWTBackend(object):

    def authenticate(self, request=None, **credentials):
        if request is None:
            return None

        token = get_authorization_header(request)
        return get_user_by_token(token)

    def get_user(self, user_id):
        return get_user_by_natural_key(user_id)



from django.contrib.auth import get_user_model
from django.test import Client, RequestFactory, testcases

from graphene_django.settings import graphene_settings
from graphql_jwt.utils import jwt_payload, jwt_encode


class GraphQLRequestFactory(RequestFactory):

    def execute(self, query, **kwargs):
        return self.schema.execute(query, variable_values=kwargs)


class GraphQLClient(GraphQLRequestFactory, Client):

    def __init__(self, **defaults):
        super().__init__(**defaults)
        self.schema = graphene_settings.SCHEMA


class GraphQLJWTTestCase(testcases.TestCase):
    client_class = GraphQLClient

    def setUp(self):
        self.user = get_user_model().objects.create_user(username='test')
        self.payload = jwt_payload(self.user)
        self.token = jwt_encode(self.payload)
        self.factory = RequestFactory()
